"use strict";
// For large dictionaries, it is necessary to increase the memory limit.
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = void 0;
const compiler_1 = require("./compiler");
const compiler = require("./compiler");
const path = require("path");
const glob = require("glob");
const gensequence_1 = require("gensequence");
const iterateWordsFromFile_1 = require("./compiler/iterateWordsFromFile");
const npmPackage = require(path.join(__dirname, '..', 'package.json'));
function globP(pattern) {
    return new Promise((resolve, reject) => {
        glob(pattern, (err, result) => {
            err ? reject(err) : resolve(result);
        });
    });
}
const log = (message, ...optionalParams) => {
    console.log(`${new Date().toISOString()} ${message}`, ...optionalParams);
};
compiler.setLogger(log);
function run(program, argv) {
    program.exitOverride();
    return new Promise((resolve, rejects) => {
        program
            .version(npmPackage.version);
        program
            .command('compile <src...>')
            .description('compile words lists into simple dictionary files.')
            .option('-o, --output <path>', 'Specify the output directory, otherwise files are written back to the same location.')
            .option('-n, --no-compress', 'By default the files are Gzipped, this will turn that off.')
            .option('-m, --max_depth <limit>', 'Maximum depth to apply suffix rules.')
            .option('-M, --merge <target>', 'Merge all files into a single target file (extensions are applied)')
            .option('-s, --no-split', 'Treat each line as a dictionary entry, do not split')
            .option('--no-sort', 'Do not sort the result')
            .action((src, options) => {
            const result = processAction(src, '.txt', options, async (src, dst) => {
                return compiler_1.compileWordList(src, dst, { splitWords: options.split, sort: options.sort }).then(() => src);
            });
            resolve(result);
        });
        program
            .command('compile-trie <src...>')
            .description('Compile words lists or Hunspell dictionary into trie files used by cspell.')
            .option('-o, --output <path>', 'Specify the output directory, otherwise files are written back to the same location.')
            .option('-m, --max_depth <limit>', 'Maximum depth to apply suffix rules.')
            .option('-M, --merge <target>', 'Merge all files into a single target file (extensions are applied)')
            .option('-n, --no-compress', 'By default the files are Gzipped, this will turn that off.')
            .option('--trie3', '[Beta] Use file format trie3')
            .action((src, options) => {
            const result = processAction(src, '.trie', options, async (words, dst) => {
                return compiler_1.compileTrie(words, dst, options);
            });
            resolve(result);
        });
        try {
            program.parse(argv);
            if (!argv.slice(2).length) {
                program.help();
            }
        }
        catch (e) {
            rejects(e);
        }
        resolve();
    });
}
exports.run = run;
function parseNumber(s) {
    const n = parseInt(s !== null && s !== void 0 ? s : '');
    return isNaN(n) ? undefined : n;
}
async function processAction(src, fileExt, options, action) {
    console.log('Compile:\n output: %s\n compress: %s\n files:\n  %s \n\n', options.output || 'default', options.compress ? 'true' : 'false', src.join('\n  '));
    const ext = fileExt + (options.compress ? '.gz' : '');
    const maxDepth = parseNumber(options.max_depth);
    const readerOptions = { maxDepth };
    const globResults = await Promise.all(src.map(s => globP(s)));
    const filesToProcess = gensequence_1.genSequence(globResults)
        .concatMap(files => files)
        .map(async (filename) => {
        log(`Reading ${path.basename(filename)}`);
        const words = await iterateWordsFromFile_1.streamWordsFromFile(filename, readerOptions);
        log(`Done reading ${path.basename(filename)}`);
        const f = {
            src: filename,
            words,
        };
        return f;
    });
    const r = options.merge
        ? processFiles(action, filesToProcess, toMergeTargetFile(options.merge, options.output, ext))
        : processFilesIndividually(action, filesToProcess, s => toTargetFile(s, options.output, ext));
    await r;
    log(`Complete.`);
}
function toFilename(name, ext) {
    return path.basename(name).replace(/((\.txt|\.dic|\.aff|\.trie)(\.gz)?)?$/, '') + ext;
}
function toTargetFile(filename, destination, ext) {
    const outFileName = toFilename(filename, ext);
    const dir = destination !== null && destination !== void 0 ? destination : path.dirname(filename);
    return path.join(dir, outFileName);
}
function toMergeTargetFile(filename, destination, ext) {
    const outFileName = toFilename(filename, ext);
    return path.resolve(destination !== null && destination !== void 0 ? destination : '.', outFileName);
}
async function processFilesIndividually(action, filesToProcess, srcToTarget) {
    const toProcess = filesToProcess
        .map(async (pFtp) => {
        const { src, words } = await pFtp;
        const dst = srcToTarget(src);
        log('Process "%s" to "%s"', src, dst);
        await action(words, dst);
        log('Done "%s" to "%s"', src, dst);
    });
    for (const p of toProcess) {
        await p;
    }
}
async function processFiles(action, filesToProcess, mergeTarget) {
    const toProcess = await Promise.all([...filesToProcess]);
    const dst = mergeTarget;
    const words = gensequence_1.genSequence(toProcess)
        .map(ftp => {
        const { src } = ftp;
        log('Process "%s" to "%s"', src, dst);
        return ftp;
    })
        .concatMap(ftp => ftp.words);
    await action(words, dst);
    log('Done "%s"', dst);
}
//# sourceMappingURL=app.js.map