"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readHunspellFiles = exports.createArrayReader = exports.createReader = void 0;
const gensequence_1 = require("gensequence");
const HR = require("hunspell-reader");
const fs = require("fs-extra");
const cspell_trie_lib_1 = require("cspell-trie-lib");
const zlib = require("zlib");
const regHunspellFile = /\.(dic|aff)$/i;
const COMPOUND = '+';
const OPTIONAL_COMPOUND = '*';
const NORMALIZED = '~';
const FORBID = '!';
// cspell:word dedupe
const DEDUPE_SIZE = 1000;
const regExMatchComments = /\s*(#|\/\/).*/;
// Readers first match wins
const readers = [
    { test: /\.trie\b/, method: trieFileReader },
    { test: regHunspellFile, method: readHunspellFiles },
];
function createReader(filename, options) {
    for (const reader of readers) {
        if (reader.test.test(filename)) {
            return reader.method(filename, options);
        }
    }
    return textFileReader(filename);
}
exports.createReader = createReader;
function createArrayReader(lines) {
    return {
        size: lines.length,
        [Symbol.iterator]: () => gensequence_1.genSequence(lines),
        annotatedWords() { return gensequence_1.genSequence(lines).pipe(_mapText).pipe(dedupeAndSort); },
    };
}
exports.createArrayReader = createArrayReader;
async function readHunspellFiles(filename, options) {
    const dicFile = filename.replace(regHunspellFile, '.dic');
    const affFile = filename.replace(regHunspellFile, '.aff');
    const reader = await HR.IterableHunspellReader.createFromFiles(affFile, dicFile);
    reader.maxDepth = options.maxDepth !== undefined ? options.maxDepth : reader.maxDepth;
    const normalizeAndDedupe = gensequence_1.operators.pipe(_stripCaseAndAccents, dedupeAndSort);
    return {
        size: reader.dic.length,
        // seqWords is used for backwards compatibility.
        [Symbol.iterator]: () => reader.seqWords(),
        annotatedWords() { return reader.seqAffWords().pipe(_mapAffWords).pipe(normalizeAndDedupe); },
    };
}
exports.readHunspellFiles = readHunspellFiles;
async function trieFileReader(filename) {
    const trieRoot = cspell_trie_lib_1.importTrie(await textFileReader(filename));
    const trie = new cspell_trie_lib_1.Trie(trieRoot);
    return {
        get size() { return trie.size(); },
        [Symbol.iterator]: () => trie.words(),
        annotatedWords() { return trie.words(); },
    };
}
async function textFileReader(filename) {
    const content = await fs.readFile(filename)
        .then(buffer => (/\.gz$/).test(filename) ? zlib.gunzipSync(buffer) : buffer)
        .then(buffer => buffer.toString('utf8'));
    const lines = content.split('\n');
    return createArrayReader(lines);
}
const _mapText = gensequence_1.operators.pipe(_comments, _compoundBegin, _compoundEnd, _stripCaseAndAccents);
function* _comments(lines) {
    for (const line of lines) {
        const w = line.replace(regExMatchComments, '').trim();
        if (w)
            yield w;
    }
}
function* _compoundEnd(lines) {
    for (const line of lines) {
        if (line[0] !== OPTIONAL_COMPOUND) {
            yield line;
            continue;
        }
        const w = line.slice(1);
        yield w;
        yield COMPOUND + w;
    }
}
function* _compoundBegin(lines) {
    for (const line of lines) {
        if (line[line.length - 1] !== OPTIONAL_COMPOUND) {
            yield line;
            continue;
        }
        const w = line.slice(0, -1);
        yield w;
        yield w + COMPOUND;
    }
}
const regNotLower = /[^a-z+!~]/;
function* _stripCaseAndAccents(words) {
    for (const word of words) {
        yield word;
        if (regNotLower.test(word)) {
            // covert to lower case and strip accents.
            const n = word.toLowerCase().normalize('NFD').replace(/[\u0300-\u036f]/g, '');
            yield NORMALIZED + n;
        }
    }
}
function* dedupeAndSort(words) {
    const buffer = [];
    function sortDedupeClear() {
        const s = new Set(buffer.sort());
        buffer.length = 0;
        return s;
    }
    for (const word of words) {
        buffer.push(word);
        if (buffer.length >= DEDUPE_SIZE) {
            yield* sortDedupeClear();
        }
    }
    yield* sortDedupeClear();
}
function* _mapAffWords(affWords) {
    for (const affWord of affWords) {
        const { word, flags } = affWord;
        const compound = flags.isCompoundForbidden ? '' : COMPOUND;
        const forbid = flags.isForbiddenWord ? FORBID : '';
        if (!forbid) {
            if (flags.canBeCompoundBegin)
                yield word + compound;
            if (flags.canBeCompoundEnd)
                yield compound + word;
            if (flags.canBeCompoundMiddle)
                yield compound + word + compound;
            if (!flags.isOnlyAllowedInCompound)
                yield word;
        }
        else {
            yield forbid + word;
        }
    }
}
//# sourceMappingURL=Reader.js.map