"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compileTrie = exports.consolidate = exports.normalizeWordsToTrie = exports.compileWordList = exports.lineToWords = exports.normalizeWords = exports.setLogger = void 0;
const cspell_util_bundle_1 = require("cspell-util-bundle");
const gensequence_1 = require("gensequence");
const Text = require("./text");
const path = require("path");
const fs_extra_1 = require("fs-extra");
const Trie = require("cspell-trie-lib");
const fileWriter_1 = require("./fileWriter");
const util_1 = require("hunspell-reader/dist/util");
const regNonWordOrSpace = cspell_util_bundle_1.xregexp("[^\\p{L}' ]+", 'gi');
const regExpSpaceOrDash = /(?:\s+)|(?:-+)/g;
const regExpRepeatChars = /(.)\1{3,}/i;
let log = defaultLogger;
function setLogger(logger) {
    log = logger !== null && logger !== void 0 ? logger : defaultLogger;
}
exports.setLogger = setLogger;
function defaultLogger(message, ...optionalParams) {
    console.log(message, ...optionalParams);
}
function normalizeWords(lines) {
    return lines.concatMap(line => lineToWords(line));
}
exports.normalizeWords = normalizeWords;
function lineToWords(line) {
    // Remove punctuation and non-letters.
    const filteredLine = line.replace(regNonWordOrSpace, '|');
    const wordGroups = filteredLine.split('|');
    const words = gensequence_1.genSequence(wordGroups)
        .concatMap(a => [a, ...a.split(regExpSpaceOrDash)])
        .concatMap(a => splitCamelCase(a))
        .map(a => a.trim())
        .filter(a => !!a)
        .filter(s => !regExpRepeatChars.test(s))
        .map(a => a.toLowerCase());
    return words;
}
exports.lineToWords = lineToWords;
function splitCamelCase(word) {
    const splitWords = Text.splitCamelCaseWord(word);
    // We only want to preserve this: "New York" and not "Namespace DNSLookup"
    if (splitWords.length > 1 && regExpSpaceOrDash.test(word)) {
        return gensequence_1.genSequence(splitWords).concatMap(w => w.split(regExpSpaceOrDash));
    }
    return splitWords;
}
async function compileWordList(words, destFilename, options) {
    const destDir = path.dirname(destFilename);
    const pDir = fs_extra_1.mkdirp(destDir);
    const compile = options.splitWords ? compileWordListWithSplitSeq : compileSimpleWordListSeq;
    const seq = compile(words)
        .filter(a => !!a)
        .filter(util_1.uniqueFilter(10000));
    const finalSeq = options.sort ? gensequence_1.genSequence(sort(seq)) : seq;
    await pDir;
    return fileWriter_1.writeSeqToFile(finalSeq.map(a => a + '\n'), destFilename);
}
exports.compileWordList = compileWordList;
function sort(words) {
    return [...words].sort();
}
function compileWordListWithSplitSeq(words) {
    return words.concatMap(line => lineToWords(line).toArray());
}
function compileSimpleWordListSeq(words) {
    return words.map(a => a.toLowerCase());
}
function normalizeWordsToTrie(words) {
    const trie = Trie.buildTrie(normalizeWords(words));
    return trie.root;
}
exports.normalizeWordsToTrie = normalizeWordsToTrie;
exports.consolidate = Trie.consolidate;
async function compileTrie(words, destFilename, options) {
    var _a;
    log('Reading Words into Trie');
    const base = (_a = options.base) !== null && _a !== void 0 ? _a : 32;
    const version = options.trie3 ? 3 : 1;
    const destDir = path.dirname(destFilename);
    const pDir = fs_extra_1.mkdirp(destDir);
    const root = normalizeWordsToTrie(words);
    log('Reduce duplicate word endings');
    const trie = exports.consolidate(root);
    log(`Writing to file ${path.basename(destFilename)}`);
    await pDir;
    await fileWriter_1.writeSeqToFile(Trie.serializeTrie(trie, { base, comment: 'Built by cspell-tools.', version }), destFilename);
    log(`Done writing to file ${path.basename(destFilename)}`);
}
exports.compileTrie = compileTrie;
//# sourceMappingURL=wordListCompiler.js.map